package model

import "github.com/shopspring/decimal"

type PaidPart struct {
	Overall        decimal.Decimal     `json:"overall"`
	OverallPrice   decimal.Decimal     `json:"overallPrice"`
	ConsumptionFee decimal.Decimal     `json:"consumptionFee"`
	OverallFee     decimal.Decimal     `json:"overallFee"`
	Critical       decimal.NullDecimal `json:"critical"`
	CriticalPrice  decimal.NullDecimal `json:"criticalPrice"`
	CriticalFee    decimal.NullDecimal `json:"criticalFee"`
	Peak           decimal.NullDecimal `json:"peak"`
	PeakPrice      decimal.NullDecimal `json:"peakPrice"`
	PeakFee        decimal.NullDecimal `json:"peakFee"`
	Flat           decimal.NullDecimal `json:"flat"`
	FlatPrice      decimal.NullDecimal `json:"flatPrice"`
	FlatFee        decimal.NullDecimal `json:"flatFee"`
	Valley         decimal.NullDecimal `json:"valley"`
	ValleyPrice    decimal.NullDecimal `json:"valleyPrice"`
	ValleyFee      decimal.NullDecimal `json:"valleyFee"`
	BasicFee       decimal.Decimal     `json:"basicFee"`
	AdjustFee      decimal.Decimal     `json:"adjustFee"`
}

type EndUserOverallPart struct {
	Overall       decimal.Decimal     `json:"overall"`
	OverallPrice  decimal.Decimal     `json:"overallPrice"`
	OverallFee    decimal.Decimal     `json:"consumptionFee"`
	Critical      decimal.NullDecimal `json:"critical"`
	CriticalPrice decimal.NullDecimal `json:"criticalPrice"`
	CriticalFee   decimal.NullDecimal `json:"criticalFee"`
	Peak          decimal.NullDecimal `json:"peak"`
	PeakPrice     decimal.NullDecimal `json:"peakPrice"`
	PeakFee       decimal.NullDecimal `json:"peakFee"`
	Flat          decimal.NullDecimal `json:"flat"`
	FlatPrice     decimal.NullDecimal `json:"flatPrice"`
	FlatFee       decimal.NullDecimal `json:"flatFee"`
	Valley        decimal.NullDecimal `json:"valley"`
	ValleyPrice   decimal.NullDecimal `json:"valleyPrice"`
	ValleyFee     decimal.NullDecimal `json:"valleyFee"`
}

type PublicConsumptionOverallPart struct {
	Overall        decimal.Decimal     `json:"overall"`
	OverallPrice   decimal.Decimal     `json:"overallPrice"`
	ConsumptionFee decimal.Decimal     `json:"consumptionFee"`
	OverallFee     decimal.Decimal     `json:"overallFee"`
	Critical       decimal.NullDecimal `json:"critical"`
	CriticalPrice  decimal.NullDecimal `json:"criticalPrice"`
	CriticalFee    decimal.NullDecimal `json:"criticalFee"`
	Peak           decimal.NullDecimal `json:"peak"`
	PeakPrice      decimal.NullDecimal `json:"peakPrice"`
	PeakFee        decimal.NullDecimal `json:"peakFee"`
	Flat           decimal.NullDecimal `json:"flat"`
	FlatPrice      decimal.NullDecimal `json:"flatPrice"`
	FlatFee        decimal.NullDecimal `json:"flatFee"`
	Valley         decimal.NullDecimal `json:"valley"`
	ValleyPrice    decimal.NullDecimal `json:"valleyPrice"`
	ValleyFee      decimal.NullDecimal `json:"valleyFee"`
	Proportion     decimal.Decimal     `json:"proportion"`
}

type LossPart struct {
	Quantity       decimal.Decimal `json:"quantity"`
	Price          decimal.Decimal `json:"price"`
	ConsumptionFee decimal.Decimal `json:"consumptionFee"`
	Proportion     decimal.Decimal `json:"proportion"`
}

type OtherShouldCollectionPart struct {
	MaintenanceFee decimal.NullDecimal `json:"maintenanceFee"`
	BasicFees      decimal.Decimal     `json:"basicFees"`
}

type MaintenancePart struct {
	BasicFees             decimal.Decimal `json:"basicFees"`
	LossFee               decimal.Decimal `json:"lossFee"`
	PublicConsumptionFee  decimal.Decimal `json:"publicConsumptionFee"`
	MaintenanceFee        decimal.Decimal `json:"maintenanceFee"`
	FinalMaintenance      decimal.Decimal `json:"finalMaintenance"`
	MaintenanceProportion decimal.Decimal `json:"maintenanceProportion"`
	MaintenancePrice      decimal.Decimal `json:"maintenancePrice"`
	PriceRatio            decimal.Decimal `json:"priceRatio"`
}

type EndUserSummary struct {
	CustomerName *string             `json:"customerName"`
	Address      *string             `json:"address"`
	MeterId      string              `json:"meterId"`
	Overall      decimal.Decimal     `json:"overall"`
	OverallFee   decimal.Decimal     `json:"overallFee"`
	Critical     decimal.NullDecimal `json:"critical"`
	CriticalFee  decimal.NullDecimal `json:"criticalFee"`
	Peak         decimal.NullDecimal `json:"peak"`
	PeakFee      decimal.NullDecimal `json:"peakFee"`
	Valley       decimal.NullDecimal `json:"valley"`
	ValleyFee    decimal.NullDecimal `json:"valleyFee"`
	Maintenance  decimal.Decimal     `json:"maintenance"`
}

type Publicity struct {
	Report                   Report                       `json:"index"`
	User                     UserDetail                   `json:"enterprise"`
	Park                     Park                         `json:"park"`
	Paid                     PaidPart                     `json:"paid"`
	EndUser                  EndUserOverallPart           `json:"endUserSum"`
	Loss                     LossPart                     `json:"loss"`
	PublicConsumptionOverall PublicConsumptionOverallPart `json:"public"`
	OtherCollections         OtherShouldCollectionPart    `json:"others"`
	Maintenance              MaintenancePart              `json:"maintenance"`
	EndUserDetails           []EndUserSummary             `json:"endUser"`
}
